searchData={"items":[{"type":"module","title":"Mockery","doc":"Core functionality","ref":"Mockery.html"},{"type":"function","title":"Mockery.mock/3","doc":"Function used to create mock in context of single test process.\n\nMock created in test won't leak to another process (other test, spawned Task, GenServer...).\nIt can be used safely in asynchronous tests.\n\nMocks can be created with static value:\n\n    mock Mod, [fun: 2], \"mocked value\"\n\nor function:\n\n    mock Mod, [fun: 2], fn(_, arg2) -> arg2 end\n\nKeep in mind that function inside mock must have same arity as\noriginal one.\n\nThis:\n\n    mock Mod, [fun: 2], &to_string/1\n\nwill raise an error.\n\nIt is also possible to mock function with given name and any arity\n\n    mock Mod, :fun, \"mocked value\"\n\nbut this version doesn't support function as value.\n\nAlso, multiple mocks for same module are chainable\n\n    Mod\n    |> mock(:fun1, \"value\")\n    |> mock([fun2: 1], &string/1)","ref":"Mockery.html#mock/3"},{"type":"function","title":"Mockery.new/2","doc":"","ref":"Mockery.html#new/2"},{"type":"function","title":"Mockery.of/2","doc":"Function used to prepare module for mocking.\n\nFor Mix.env other than :test it returns module given in the first argument.\nIf Mix.env equal :test it creates a proxy to the original module.\nWhen Mix is missing it assumes that env is :prod","ref":"Mockery.html#of/2"},{"type":"function","title":"Examples - Mockery.of/2","doc":"#### Prepare for mocking (elixir module)\n\n    defmodule Foo do\n      @bar Mockery.of(\"Bar\")\n\n      def foo do\n        @bar.bar()\n      end\n    end\n\nIt is also possible to pass the module in elixir format\n\n    @bar Mockery.of(Bar)\n\nbut it is not recommended as it creates an unnecessary compile-time dependency\n(see `mix xref graph` output for both versions).\n\n#### Prepare for mocking (erlang module)\n\n    defmodule Foo do\n      @crypto Mockery.of(:crypto)\n\n      def foo do\n        @crypto.rand_seed()\n      end\n    end\n\n#### Prepare for mocking with global mock\n\n    # test/support/global_mocks/bar.ex\n    defmodule BarGlobalMock do\n      def bar, do: :mocked\n    end\n\n    # lib/foo.ex\n    defmodule Foo do\n      @bar Mockery.of(Bar, by: BarGlobalMock)\n\n      def foo do\n        @bar.bar()\n      end\n    end","ref":"Mockery.html#of/2-examples"},{"type":"function","title":"OTP21+ - Mockery.of/2","doc":"Internally mockery is using tuple calls to pass additional data to its proxy module\nwhen mock is called. Tuple calls are disabled by default in OTP21+ and require additional\ncompile flag to be reenabled.\n\n    defmodule Foo do\n      @compile :tuple_calls\n      @bar Mockery.of(\"Bar\")\n\n      # ...\n    end\n\nIf you don't want to reenable tuple calls, there's also new macro-based alternative\n(for more information see `Mockery.Macro` module).","ref":"Mockery.html#of/2-otp21"},{"type":"type","title":"Mockery.elixir_module_as_string/0","doc":"Used to avoid unnecessary compile-time dependencies between modules","ref":"Mockery.html#t:elixir_module_as_string/0"},{"type":"type","title":"Examples - Mockery.elixir_module_as_string/0","doc":"defmodule Foo do\n      # this creates compile-time dependency between Foo and Bar\n      @bar1 Mockery.of(Bar)\n\n      # same result but without compile-time dependency\n      @bar2 Mockery.of(\"Bar\")\n    end\n\n`mix xref graph` can be used to check difference between module and string versions","ref":"Mockery.html#t:elixir_module_as_string/0-examples"},{"type":"opaque","title":"Mockery.proxy_tuple/0","doc":"Mockery uses tuple calls to send additional data to internal proxy module","ref":"Mockery.html#t:proxy_tuple/0"},{"type":"module","title":"Mockery.Assertions","doc":"Set of additional assertion functions.\n\n**NOTE:** Mockery doesn't keep track of function calls:\n\n  * on modules that weren't prepared by `Mockery.of/2` or `Mockery.Macro.mockable/2`\n\n  * for `MIX_ENV` other than `:test`\n\n  * for function called outside of test process (in spawned Task, GenServer etc.)","ref":"Mockery.Assertions.html"},{"type":"function","title":"Mockery.Assertions.assert_called/2","doc":"Asserts that function from given module with given name or name and arity\nwas called at least once.","ref":"Mockery.Assertions.html#assert_called/2"},{"type":"function","title":"Examples - Mockery.Assertions.assert_called/2","doc":"Assert Mod.fun/2 was called\n\n    assert_called Mod, fun: 2\n\nAssert any function named :fun from module Mod was called\n\n    assert_called Mod, :fun","ref":"Mockery.Assertions.html#assert_called/2-examples"},{"type":"macro","title":"Mockery.Assertions.assert_called/3","doc":"Asserts that function from given module with given name was called\nat least once with arguments matching given pattern.","ref":"Mockery.Assertions.html#assert_called/3"},{"type":"macro","title":"Examples - Mockery.Assertions.assert_called/3","doc":"Assert Mod.fun/2 was called with given args list\n\n    assert_called Mod, :fun, [\"a\", \"b\"]\n\nYou can also use unbound variables inside args pattern\n\n    assert_called Mod, :fun, [\"a\", _second]","ref":"Mockery.Assertions.html#assert_called/3-examples"},{"type":"macro","title":"Mockery.Assertions.assert_called/4","doc":"Asserts that function from given module with given name was called\ngiven number of times with arguments matching given pattern.\n\nSimilar to `assert_called/3` but instead of checking if function was called\nat least once, it checks if function was called specific number of times.","ref":"Mockery.Assertions.html#assert_called/4"},{"type":"macro","title":"Examples - Mockery.Assertions.assert_called/4","doc":"Assert Mod.fun/2 was called with given args 5 times\n\n    assert_called Mod, :fun, [\"a\", \"b\"], 5\n\nAssert Mod.fun/2 was called with given args from 3 to 5 times\n\n    assert_called Mod, :fun, [\"a\", \"b\"], 3..5\n\nAssert Mod.fun/2 was called with given args 3 or 5 times\n\n    assert_called Mod, :fun, [\"a\", \"b\"], [3, 5]","ref":"Mockery.Assertions.html#assert_called/4-examples"},{"type":"function","title":"Mockery.Assertions.refute_called/2","doc":"Asserts that function from given module with given name or name and arity\nwas NOT called.","ref":"Mockery.Assertions.html#refute_called/2"},{"type":"function","title":"Examples - Mockery.Assertions.refute_called/2","doc":"Assert Mod.fun/2 wasn't called\n\n    refute_called Mod, fun: 2\n\nAssert any function named :fun from module Mod wasn't called\n\n    refute_called Mod, :fun","ref":"Mockery.Assertions.html#refute_called/2-examples"},{"type":"macro","title":"Mockery.Assertions.refute_called/3","doc":"Asserts that function from given module with given name was NOT called\nwith arguments matching given pattern.","ref":"Mockery.Assertions.html#refute_called/3"},{"type":"macro","title":"Examples - Mockery.Assertions.refute_called/3","doc":"Assert Mod.fun/2 wasn't called with given args list\n\n    refute_called Mod, :fun, [\"a\", \"b\"]\n\nYou can also use unbound variables inside args pattern\n\n    refute_called Mod, :fun, [\"a\", _second]","ref":"Mockery.Assertions.html#refute_called/3-examples"},{"type":"macro","title":"Mockery.Assertions.refute_called/4","doc":"Asserts that function from given module with given name was NOT called\ngiven number of times with arguments matching given pattern.\n\nSimilar to `refute_called/3` but instead of checking if function was called\nat least once, it checks if function was called specific number of times.","ref":"Mockery.Assertions.html#refute_called/4"},{"type":"macro","title":"Examples - Mockery.Assertions.refute_called/4","doc":"Assert Mod.fun/2 was not called with given args 5 times\n\n    refute_called Mod, :fun, [\"a\", \"b\"], 5\n\nAssert Mod.fun/2 was not called with given args from 3 to 5 times\n\n    refute_called Mod, :fun, [\"a\", \"b\"], 3..5\n\nAssert Mod.fun/2 was not called with given args 3 or 5 times\n\n    refute_called Mod, :fun, [\"a\", \"b\"], [3, 5]","ref":"Mockery.Assertions.html#refute_called/4-examples"},{"type":"module","title":"Mockery.History","doc":"Provides calls history for Mockery.Assertions macros.\n\nIt's disabled by default.\nIt can be enabled/disabled globally by following config\n\n    config :mockery, history: true\n\nOr for single test process\n\n    Mockery.History.enable_history()\n    Mockery.History.disable_history()\n\nProcess config has higher priority than global config","ref":"Mockery.History.html"},{"type":"function","title":"Mockery.History.disable_history/0","doc":"Disables history in scope of single test process\n\n    use Mockery\n\n    test \"example\" do\n      #...\n\n      disable_history()\n      assert_called Foo, :bar, [_, :a]\n    end","ref":"Mockery.History.html#disable_history/0"},{"type":"function","title":"Mockery.History.enable_history/0","doc":"Enables history in scope of single test process\n\n    use Mockery\n\n    test \"example\" do\n      #...\n\n      enable_history()\n      assert_called Foo, :bar, [_, :a]\n    end","ref":"Mockery.History.html#enable_history/0"},{"type":"function","title":"Mockery.History.enable_history/1","doc":"","ref":"Mockery.History.html#enable_history/1"},{"type":"module","title":"Mockery.Macro","doc":"Alternative macro-based way to prepare module for mocking/asserting.","ref":"Mockery.Macro.html"},{"type":"macro","title":"Mockery.Macro.mockable/2","doc":"Function used to prepare module for mocking/asserting.\n\nFor Mix.env other than :test it returns the first argument unchanged.\nIf Mix.env equal :test it creates a proxy to the original module.\nWhen Mix is missing it assumes that env is :prod.","ref":"Mockery.Macro.html#mockable/2"},{"type":"macro","title":"Examples - Mockery.Macro.mockable/2","doc":"#### Prepare for mocking\n\n    defmodule Foo do\n      import Mockery.Macro\n\n      def foo do\n        mockable(Bar).bar()\n      end\n    end\n\n#### Prepare for mocking with global mock\n\n    # test/support/global_mocks/bar.ex\n    defmodule BarGlobalMock do\n      def bar, do: :mocked\n    end\n\n    # lib/foo.ex\n    defmodule Foo do\n      import Mockery.Macro\n\n      def foo do\n        mockable(Bar, by: BarGlobalMock).bar()\n      end\n    end","ref":"Mockery.Macro.html#mockable/2-examples"},{"type":"macro","title":"Mockery.of/2 comparison - Mockery.Macro.mockable/2","doc":"* It's based on macro and process dictionary instead of on tuple calls. (Tuple calls\n  are disabled by default in OTP21+ and require additional compile flag to be reenabled)\n  * It doesn't support passing module names as a string as it don't create unwanted compile-time\n  dependencies between modules","ref":"Mockery.Macro.html#mockable/2-mockery-of-2-comparison"},{"type":"macro","title":"Potential issues - Mockery.Macro.mockable/2","doc":"Output of `mockable/2` macro should not be bind to variable or module attribute.\nIf it happens, you'll see a compilation warning at best, and in the worst case Mockery won't\nwork correctly.","ref":"Mockery.Macro.html#mockable/2-potential-issues"},{"type":"exception","title":"Mockery.Error","doc":"","ref":"Mockery.Error.html"},{"type":"extras","title":"Mockery","doc":"# Mockery\n\n[![Build Status](https://github.com/appunite/mockery/workflows/Test/badge.svg?branch=master)](https://github.com/appunite/mockery/actions) [![SourceLevel](https://app.sourcelevel.io/github/appunite/-/mockery.svg)](https://app.sourcelevel.io/github/appunite/-/mockery) [![Hex.pm](https://img.shields.io/hexpm/v/mockery.svg)](https://hex.pm/packages/mockery) [![Hex.pm](https://img.shields.io/hexpm/dt/mockery.svg)](https://hex.pm/packages/mockery) [![Hex.pm](https://img.shields.io/hexpm/dw/mockery.svg)](https://hex.pm/packages/mockery)\n\nSimple mocking library for asynchronous testing in Elixir.\n\n> Readme and documentation for last stable version are available on [hex](https://hexdocs.pm/mockery/readme.html)","ref":"readme.html"},{"type":"extras","title":"Advantages - Mockery","doc":"- Mockery does not override your modules\n- Mockery does not replace modules by aliasing\n- Mockery does not require to pass modules as function parameter\n- Mockery does not require to create callbacks or wrappers around libraries\n- Mockery does not create modules during runtime (neither by `defmodule/2` nor `Module.create/3`)\n- Mockery does not allow to mock non-existent function\n- Mockery does not share any data between test processes","ref":"readme.html#advantages"},{"type":"extras","title":"Disadvantages - Mockery","doc":"- Mockery is not designed for libraries as it would force end user to download Mockery as dependency of dependency\n- Mockery can cause issues with dialyzer if you are using dialyzer with `MIX_ENV=test`","ref":"readme.html#disadvantages"},{"type":"extras","title":"Upgrading Erlang/OTP to 21 - Mockery","doc":"Erlang/OTP 21 contains some changes that prevent the package from functioning as before. \nPlease check [migration guide](MIGRATION_TO_OTP21.md).","ref":"readme.html#upgrading-erlang-otp-to-21"},{"type":"extras","title":"Getting started - Mockery","doc":"","ref":"readme.html#getting-started"},{"type":"extras","title":"Installation - Mockery","doc":"```elixir\ndef deps do\n  [\n    {:mockery, \"~> 2.3.0\", runtime: false}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Preparation of the module for mocking - Mockery","doc":"```elixir\n# lib/my_app/foo.ex\ndefmodule MyApp.Foo do\n  import Mockery.Macro\n  alias MyApp.Bar\n\n  def baz, do: mockable(Bar).function()\nend\n```","ref":"readme.html#preparation-of-the-module-for-mocking"},{"type":"extras","title":"Basic usage - Mockery","doc":"","ref":"readme.html#basic-usage"},{"type":"extras","title":"Static value mock - Mockery","doc":"```elixir\ndefmodule MyApp.Controller do\n  # ...\n  import Mockery.Macro\n\n  def all do\n    mockable(MyApp.UserService).users()\n  end\n\n  def filtered do\n    mockable(MyApp.UserService).users(\"filter\")\n  end\nend\n\n# tests\ndefmodule MyApp.ControllerTest do\n  # ...\n  import Mockery\n\n  test \"mock any function :users from MyApp.UserService\" do\n    mock MyApp.UserService, :users, \"mock\"\n    assert all() == \"mock\"\n    assert filtered() == \"mock\"\n  end\n\n  test \"mock MyApp.UserService.users/0\" do\n    mock MyApp.UserService, [users: 0], \"mock\"\n    assert all() == \"mock\"\n    refute filtered() == \"mock\"\n  end\n\n  test \"mock MyApp.UserService.users/0 with default value\" do\n    mock MyApp.UserService, users: 0\n    assert all() == :mocked\n    refute filtered() == :mocked\n  end\n\n  test \"chaining multiple mocks for same module\" do\n    UserService\n    |> mock([users: 0], \"mock value\")\n    |> mock([users: 1], \"mock value\")\n    # ...\n  end\nend\n```","ref":"readme.html#static-value-mock"},{"type":"extras","title":"Dynamic mock - Mockery","doc":"Instead of using a static value, you can use a function with the same arity as original one.\n\n```elixir\ndefmodule Foo do\n  def bar(value), do: value\nend\n\n# prepare tested module\ndefmodule Other do\n  import Mockery.Macro\n\n  def parse(value) do\n    mockable(Foo).bar(value)\n  end\nend\n\n# tests\ndefmodule OtherTest do\n # ...\n import Mockery\n\n  test \"with dynamic mock\" do\n    mock Foo, [bar: 1], fn(value)-> String.upcase(value) end\n    assert parse(\"test\") == \"TEST\"\n  end\nend\n```","ref":"readme.html#dynamic-mock"},{"type":"extras","title":"Checking if function was called - Mockery","doc":"```elixir\n# prepare tested module\ndefmodule Tested do\n  import Mockery.Macro\n\n  def call(value, opts) do\n    mockable(Foo).bar(value)\n  end\nend\n\n# tests\ndefmodule TestedTest do\n  # ...\n  import Mockery.Assertions\n  # use Mockery # when you need to import both Mockery and Mockery.Assertions\n\n  test \"assert any function bar from module Foo was called\" do\n    Tested.call(1, %{})\n    assert_called Foo, :bar\n  end\n\n  test \"assert Foo.bar/2 was called\" do\n    Tested.call(1, %{})\n    assert_called Foo, bar: 2\n  end\n\n  test \"assert Foo.bar/2 was called with given args\" do\n    Tested.call(1, %{})\n    assert_called Foo, :bar, [1, %{}]\n  end\n\n  test \"assert Foo.bar/1 was called with given arg (using variable)\" do\n    params = %{a: 1, b: 2}\n\n    Tested.call(params)\n    assert_called Foo, :bar, [^params]\n    # we need to use pinning here since assert_called/3 is a macro\n    # and not a regular function call and it gets expanded accordingly\n  end\n\n  test \"assert Foo.bar/2 was called with 1 as first arg\" do\n    Tested.call(1, %{})\n    assert_called Foo, :bar, [1, _]\n  end\n\n  test \"assert Foo.bar/2 was called with 1 as first arg 5 times\" do\n    # ...\n    assert_called Foo, :bar, [1, _], 5\n  end\n\n  test \"assert Foo.bar/2 was called with 1 as first arg from 3 to 5 times\" do\n    # ...\n    assert_called Foo, :bar, [1, _], 3..5\n  end\n\n  test \"assert Foo.bar/2 was called with 1 as first arg 3 or 5 times\" do\n    # ...\n    assert_called Foo, :bar, [1, _], [3, 5]\n  end\nend\n```","ref":"readme.html#checking-if-function-was-called"},{"type":"extras","title":"Refute - Mockery","doc":"Every assert_called/x function/macro has its refute_called/x counterpart. \nFor more information see [docs](https://hexdocs.pm/mockery/Mockery.Assertions.html)","ref":"readme.html#refute"},{"type":"extras","title":"History - Mockery","doc":"![history example](https://raw.githubusercontent.com/appunite/mockery/master/history.jpeg)\n\nMockery.History module provides more descriptive failure messages for assert_called/{3,4} and refute_called/{3,4} that includes a colorized list of arguments passed to a given function in the scope of a single test process.\n\nDisabled by default. For more information see [docs](https://hexdocs.pm/mockery/Mockery.History.html)","ref":"readme.html#history"},{"type":"extras","title":"Global mock - Mockery","doc":"Useful when you need to use the same mock many times across different tests\n\n```elixir\ndefmodule Foo do\n  def bar, do: 1\n  def baz, do: 2\nend\n\ndefmodule FooGlobalMock do\n  def bar, do: :mocked\nend\n\n# prepare tested module\ndefmodule Other do\n  import Mockery.Macro\n\n  def bar, do: mockable(Foo, by: FooGlobalMock).bar()\n  def baz, do: mockable(Foo, by: FooGlobalMock).baz()\nend\n\n# tests\ndefmodule OtherTest do\n  # ...\n\n  test \"with global mock\" do\n    assert Other.bar == :mocked\n    assert Other.baz == 2\n  end\nend\n```","ref":"readme.html#global-mock"},{"type":"extras","title":"Restrictions - Mockery","doc":"Global mock module doesn't have to contain every function exported by the original module, but it cannot contain a function which is not exported by the original module. \nIt means that:\n\n- when you remove a function from the original module, you have to remove it from global mock module or Mockery will raise exception\n- when you change a function name in the original module, you have to change it in global mock module or Mockery will raise exception\n- when you change a function arity in the original module, you have to change it in global mock module or Mockery will raise exception","ref":"readme.html#restrictions"},{"type":"extras","title":"Advanced examples - Mockery","doc":"For advanced usage examples see","ref":"readme.html#advanced-examples"},{"type":"extras","title":"External resources - Mockery","doc":"-","ref":"readme.html#external-resources"},{"type":"extras","title":"License - Mockery","doc":"Copyright 2017-2019 Tobiasz Małecki [tobiasz.malecki@appunite.com](mailto:tobiasz.malecki@appunite.com)\n\nLicensed under the Apache License, Version 2.0 (the \"License\"); you may not use this file except in compliance with the License. You may obtain a copy of the License at\n\n \n\nUnless required by applicable law or agreed to in writing, software distributed under the License is distributed on an \"AS IS\" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"readme.html#license"},{"type":"extras","title":"Examples","doc":"# Examples","ref":"examples.html"},{"type":"extras","title":"Situations triggered by another process - Examples","doc":"Let's assume we have a unique resource and function that is fetching it if exist or create new otherwise.\n\n```elixir\n  @service Mockery.of(\"Service\")\n\n  def fetch_or_create do\n    with \\\n      {:error, :not_found} <- @service.fetch(),\n      {:ok, resource} <- @service.create()\n    do\n      {:ok, resource}\n    else\n      {:ok, resource} ->\n        {:ok, resource}\n      {:error, %Changeset{errors: [identifier: \"has already been taken\"]}} ->\n        fetch_or_create()\n    end\n  end\n```\n\n`{:error, %Changeset{}}` in else clause can match only if between `@service.fetch` and `@service.create` there was another process that successfully created resource (`Service.create()`)\n\nHow to cover `{:error, %Changeset{}} -> fetch_or_create()` with tests?\n\n```elixir\nmock Service, :create, {:error, %Ecto.Changeset{...}}\nmock Service, [fetch: 0], fn ->\n  mock Service, [fetch: 0], &Service.fetch/0\n\n  {:error, :not_found}\nend\n\nassert {:ok, %Resource{}} = fetch_or_create()\n```\n\nIn solution above:\n\n- `Service.create/0` is permanently mocked to return `{:error, %Changeset{}}`\n- First call of `Service.fetch/0` changes mock for next calls to original function and returns `{:error, :not_found}`\n- Second call of `Service.fetch/0` returns `{:ok, resource}`","ref":"examples.html#situations-triggered-by-another-process"},{"type":"extras","title":"Task.Supervisor - Examples","doc":"Run tasks synchronously in test environment for easier testing\n\n```elixir\n  defmodule MyApp.Service do\n    def something, do: \"value\"\n  end\n```\n\n```elixir\n  defmodule MyApp.Controller do\n    @task_supervisor Mockery.of(\"Task.Supervisor\")\n    @service Mockery.of(\"MyApp.Service\")\n\n    def action do\n      @task_supervisor.start_child(MyApp.TaskSupervisor, fn->\n        @service.something()\n      end)\n    end\n  end\n```\n\n```elixir\n  defmodule MyApp.ControllerTest do\n    use ExUnit.Case, async: true\n    import Mockery.Assertions\n\n    test \"something is called\" do\n      mock Task.Supervisor, :start_child, fn(_, fun) -> fun.() end\n      MyApp.Controller.action()\n\n      assert_called MyApp.Service, :something\n    end\n  end\n```","ref":"examples.html#task-supervisor"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"2.3.2 (2024-07-03) - Changelog","doc":"- Added missing () to fix warnings","ref":"changelog.html#2-3-2-2024-07-03"},{"type":"extras","title":"2.3.1 (2019-10-10) - Changelog","doc":"- Fixed issue preventing module attributes to be used in assert_called macros [#34](https://github.com/appunite/mockery/pull/34)","ref":"changelog.html#2-3-1-2019-10-10"},{"type":"extras","title":"2.3.0 (2018-10-22) - Changelog","doc":"- Allowed multiple mocks when using pipe operator or nested calls [#27](https://github.com/appunite/mockery/pull/27)\n- Deprecated `Mockery.History.enable_history/1` in favor of `Mockery.History.enable_history/0` and `Mockery.History.disable_history/0`","ref":"changelog.html#2-3-0-2018-10-22"},{"type":"extras","title":"2.2.0 (2018-06-17) - Changelog","doc":"- Added `Mockery.Macro.mockable/2` macro as alternative way to prepare module for mocking\n- Deprecated `Mockery.new/2`","ref":"changelog.html#2-2-0-2018-06-17"},{"type":"extras","title":"2.1.0 (2018-02-19) - Changelog","doc":"- Added `Mockery.new/2`\n- Fixed typo in error message [#21](https://github.com/appunite/mockery/pull/21)\n- Removed support for Elixir 1.1 and 1.2","ref":"changelog.html#2-1-0-2018-02-19"},{"type":"extras","title":"2.0.2 (2017-10-25) - Changelog","doc":"- Fixed `Mockery.of/2` typespecs","ref":"changelog.html#2-0-2-2017-10-25"},{"type":"extras","title":"2.0.1 (2017-10-13) - Changelog","doc":"- When Mix is missing, `Mockery.of/2` assumes that env is `:prod`","ref":"changelog.html#2-0-1-2017-10-13"},{"type":"extras","title":"2.0.0 (2017-10-08) - Changelog","doc":"- Removed `Mockery.Heritage`\n\n  - Global mocks will be handled without macros by pure elixir modules.\n  - Some new restrictions have been added for global mock modules (see global mock section in README)\n\n- Changed `Mockery.mock/3` when value is function (dynamic mock)\n\n  - `[function_name: arity]` syntax remains unchanged\n  - `:function_name` syntax will raise error\n\n- Changed `Mockery.of/2` output\n\n  - run `mix compile --force` when upgrading from Mockery 1.x.x","ref":"changelog.html#2-0-0-2017-10-08"},{"type":"extras","title":"1.4.0 (2017-10-02) - Changelog","doc":"- `Mockery.mock/2` and `Mockery.mock/3` are now chainable","ref":"changelog.html#1-4-0-2017-10-02"},{"type":"extras","title":"1.3.1 (2017-09-21) - Changelog","doc":"- Minor format fix in `Mockery.History` failure messages","ref":"changelog.html#1-3-1-2017-09-21"},{"type":"extras","title":"1.3.0 (2017-09-15) - Changelog","doc":"- Added string version for `Mockery.of/2` (now recommended)\n- Fixed mocking of erlang modules","ref":"changelog.html#1-3-0-2017-09-15"},{"type":"extras","title":"1.2.0 (2017-09-13) - Changelog","doc":"- Introduced `Mockery.History`","ref":"changelog.html#1-2-0-2017-09-13"},{"type":"extras","title":"1.1.1 (2017-09-05) - Changelog","doc":"- More descriptive errors (when args are not list) for:\n\n  - `Mockery.Assertions.assert_called/3`\n  - `Mockery.Assertions.refute_called/3`\n  - `Mockery.Assertions.assert_called/4`\n  - `Mockery.Assertions.refute_called/4`\n\n- Removed unnecessary `Elixir.` namespace from module names in error messages","ref":"changelog.html#1-1-1-2017-09-05"},{"type":"extras","title":"1.1.0 (2017-08-07) - Changelog","doc":"- Added `Mockery.mock/2` (`Mockery.mock/3` value defaults to `:mocked`)\n- Added `Mockery.Assertions.assert_called/4` macro\n- Added `Mockery.Assertions.refute_called/4` macro","ref":"changelog.html#1-1-0-2017-08-07"},{"type":"extras","title":"1.0.1 (2017-08-02) - Changelog","doc":"- Fixed issue when mocking with `nil` or `false`","ref":"changelog.html#1-0-1-2017-08-02"},{"type":"extras","title":"1.0.0 (2017-07-30) - Changelog","doc":"- Added `use Mockery` for importing both `Mockery` and `Mockery.Assertions`\n- Added `Mockery.Assertions.refute_called/2` and `Mockery.Assertions.refute_called/3`\n- Loosen elixir version requirement from `~> 1.3` to `~> 1.1`","ref":"changelog.html#1-0-0-2017-07-30"},{"type":"extras","title":"Migration to Erlang/OTP21","doc":"# Migration to Erlang/OTP21","ref":"migration_to_otp21.html"},{"type":"extras","title":"Tuple calls - Migration to Erlang/OTP21","doc":"Due to this PR  , it is now necessary to add\n`@compile :tuple_calls` in every module where old Mockery API was used.\n\nSo, for example, most efficient way to make this\n\n```elixir\ndefmodule MyProject do\n  @foo Mockery.of(\"Foo\")\n\n  def bar, do: @foo.bar()\n  def baz, do: @foo.baz()\nend\n```\n\nworks as expected after OTP upgrade is\n\n```elixir\ndefmodule TupleCalls do\n  defmacro __using__(_opts) do\n    if function_exported?(Mix, :env, 0) && Mix.env() == :test do\n      quote do: @compile(:tuple_calls)\n    end\n  end\nend\n\ndefmodule MyProject do\n  use TupleCalls\n\n  @foo Mockery.of(\"Foo\")\n\n  def bar, do: @foo.bar()\n  def baz, do: @foo.baz()\nend\n```\n\nThis reenables tuple calls only for `:test` environment.","ref":"migration_to_otp21.html#tuple-calls"},{"type":"extras","title":"Macro-based alternative - Migration to Erlang/OTP21","doc":"For those who prefer not to reenable tuple calls, there is a new macro-based API.\n\nPrevious example rewritten to use macros:\n\n```elixir\ndefmodule MyProject do\n  import Mockery.Macro\n\n  def bar, do: mockable(Foo).bar()\n  def baz, do: mockable(Foo).baz()\nend\n```","ref":"migration_to_otp21.html#macro-based-alternative"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}